#!/usr/bin/perl -wT

=head1 NAME

comicman.pl

=cut

=head1 SYNOPSIS

web comic downloading system

=cut

=head1 DESCRIPTION

ComicMan is a tool to download webcomics. It comes with both command line
and graphical tools for its own configuration.

=cut

=head1 AUTHOR

Jay Scott Raymond

=cut

=head1 BUGS

None known as yet.

=cut

=head1 SEE ALSO

inf-doc.txt

=cut

=head1 COPYRIGHT

Copyright 2006, Jay Scott Raymond. All Rights Reserved

This program is free software. You may copy or
redistribute it under the same terms as Perl itself.

=cut

use strict;
use warnings;
use Getopt::Long;
use lib '/usr/share/comicman/pm';
use Comiclib;

delete @ENV{qw(IFS CDPATH ENV BASH_ENV)};   # Make %ENV safer
$ENV{'PATH'}="/usr/bin:/bin";

#set_cm_data('debug_level', 1);
#set_cm_data('log_setting', 0);

our $VERSION = '0.0.3';

my $inf_name;
my @inf_names;
my $the_page;
my %inf_data;
my @comics;
my @arguments=@ARGV;
my $num_args=@ARGV;
my $cmd_arg;
my @cmd_infs;
my @valid_inf;
my $comicman_version="0.0.3";
my $accepted_inf_ver="0.0.1";
my $new_inf=0;
my $setup=0;
my $cmd_inf = 0;
my $debug_level;
my $network_timeout;
my $log_setting;
#my $user_conf;
my $conf_check;
#my @sys_conf_fields=('sysinf_files_dir', 'log_setting', 'debug_level', 'network_timeout');
#my @user_conf_fields=('log_setting', 'debug_level', 'network_timeout');
my @required_fields=('infVersion','comicTitle','pageURL',
                   'regEx','comicURL','datFile','comicType');

# OS compatibility warning
if ( $^O ne 'linux' ) {
	debug_print(3, "WARNING: This software has not been tested on operating systems other than Linux!\n");
}

# check config files
$conf_check = init_cm_data();
if( $conf_check == 0) {
	debug_print(1, "Problem with: conf files. Please check them. Quitting.\n");
	exit;
}

$network_timeout = get_cm_data('network_timeout');
$debug_level = get_cm_data('debug_level');
$log_setting = get_cm_data('log_setting');

# handle command line options

=head2 Command line options:

-debug <debug level (0-4)>  EX: comicman.pl -debug 4

-logging <0=off, 1=on>  EX: comicman.pl -logging 1

-newinf : generate new inf file

-setup : first time user setup and choosing your comics

-timeout: timeout in seconds for network operations Valid values: 5-30.

<inf file name(s)> : get these comic(s) only  EX: comicman.pl kevin-kell.inf cotc.inf

=cut

GetOptions("debug=i" => \$debug_level,
           "logging=i" =>\$log_setting,
           "newinf" =>\$new_inf,
           "setup" =>\$setup,
           "timeout=i" => \$network_timeout);
if (($debug_level ge 0) && ($debug_level le 4) ){
	set_cm_data('debug_level', $debug_level);
} else {
	print "Ignoring invalid debug setting. Valid: 0-4.\n";
}
if( ($log_setting == 0) || ($log_setting == 1) ) {
	set_cm_data('log_setting', $log_setting);
} else {
	print "Ignoring invalid logging setting. Valid: 0-1.\n";
}
if (($network_timeout >= 5) && ($network_timeout <= 30) ) {
	set_cm_data('network_timeout', $network_timeout);
} else {
	print "Ignoring invalid timeout setting. Valid: 5-30.\n";
}
if ($new_inf) {
	make_new_inf();
	exit;
}
if ($setup) {
	comicman_setup();
	exit;
}

# get list of inf files
@inf_names=get_chosen_list();
if ( scalar(@inf_names) == 0 ) {
	print "Please run \"cmconfig.pl\" (preferred) or \"comicman.pl -setup\"\n";
	exit;
}

# validate command line infs (if any) against list of infs
if ( $num_args > 0 ) {
	foreach $cmd_arg (@arguments) {
		chomp $cmd_arg;
		if ( $cmd_arg=~m%[a-zA-Z0-9\-]*\.inf%) {	
			# arg is a .inf
			$cmd_inf = 1;
			@valid_inf = grep(m%$cmd_arg%, @inf_names);
			if ( scalar(@valid_inf) > 0 ) {
				# arg is a valid .inf
				push( @cmd_infs, $cmd_arg );
			}
			else {
				debug_print(1,  "unknown file: $cmd_arg\n");
			}
		}
	}
	if ( scalar(@cmd_infs) > 0 ) {
		@inf_names = @cmd_infs;
	}
}
# if infs were passed on the command line, but none were valid
# then exit instead of running through all chosen infs
if (($cmd_inf == 1) && (scalar(@cmd_infs) == 0)) {
	exit;
}

foreach $inf_name (@inf_names) {
	%inf_data=fetch_inf_data($inf_name);

	if (defined ($inf_data{fail}) ){
		debug_print(1,  "fetch_inf_data failed\n");
		exit;
	}
	if (!defined ($inf_data{referer}) ){
		$inf_data{referer} = "none";
	}
	if ($inf_data{infVersion} ne $accepted_inf_ver) {
		debug_print(1,  "bad inf version in $inf_name\n");
		debug_print(1,  "hdata: $inf_data{infVersion}\n");
		debug_print(1,  "okver: $accepted_inf_ver\n");
		next;
	}
	if (missing_fields(\@required_fields,\%inf_data)) {
		debug_print(1, "missing fields in $inf_name\n");
		next;
	}
	print "$inf_data{comicTitle}\n";
	debug_print(1, "$inf_data{comicTitle}\n");
	# get the webpage
	$the_page = get_webpage ($inf_data{pageURL});
	if ( (length($the_page) < 10) || ($the_page eq -1)  ) {
		debug_print(1, "\tfailed to get webpage: $inf_data{pageURL}\n"); 
		next;
	}
	# parse out the filename
	@comics = parse_filename( $the_page, "$inf_data{regEx}" );
        if (defined($inf_data{fixUpSearch}) && defined($inf_data{fixUpReplace})) {
		if (($inf_data{fixUpSearch} ne 'none') && ($inf_data{fixUpReplace} ne 'none')) {
			@comics = name_fixup($inf_data{fixUpSearch},$inf_data{fixUpReplace},\@comics);
		}
	}
	# get the comic(s)
	if ($inf_data{comicType} eq "named" ) {
		get_named_comics( $inf_data{referer},$inf_data{comicURL}, $inf_data{datFile}, \@comics );
	} elsif ($inf_data{comicType} eq "checksum" ) {
		get_cksum_comics($inf_data{referer},$inf_data{comicURL},$inf_data{datFile},\@comics);
	} else {
		debug_print(1, print "unknown comic type: $inf_data{comicType}\n");
	}
	if (defined($inf_data{renameIt}) ) {
		if ( $inf_data{renameIt} eq "prefix" ) {
			prefix_comic($inf_data{prefix}, \@comics);
		}elsif ($inf_data{renameIt} eq "full") {
			rename_comic($inf_data{nameSchema},$inf_data{newName},\@comics)
		}
	}
	undef %inf_data;
}
